// poi: find & delete target file
// Copyright (C) 2004 GORRY.

#include "poi.h"

typedef struct tagPOITHREADPARAM {
	OPTION_FOLDER *option;
	int testflag;
	HWND hwndnotify;
} POITHREADPARAM;

static int Poi_Busy;
static int Poi_VerboseLog;
static int Poi_CompareTime;
static LPTSTR Poi_CompareTimeStr;

////////////////////////////////////////////////////////////////
// tH_XLăc[\z

static int CreateFolderNameTbl(
	FOLDERNAMETBL *foldernametbl,
	FILELIST *filelist,
	BOOL findsubfolder,
	TCHAR *targetext
) {
	int ret;
	HANDLE hfind = NULL;
	WIN32_FIND_DATA finddata;
	FILEENTRY *f;
	DWORD fileentrycount;
	FOLDERNAMETBL *ft;
	FOLDERNAMETBL *ft_prev = NULL;
	TCHAR path[MAX_PATH*2+1];
	int lasterror = 0;

	if (Poi_VerboseLog) LOG_Put_S1(TEXT("CreateFolderNameTbl(): start: path=[%s]\n"), foldernametbl->Path );

	if ( filelist->Entry == NULL ) {
		void *p;
		DWORD size;
		filelist->EntryCount = 0;
		size = DEFSIZE_FILEENTRY;
		p = malloc( sizeof(FILEENTRY) * size );
		if ( p == NULL ) {
			LOG_Put_I1(TEXT("CreateFolderNameTbl(): failed: malloc filelist->Entry*%d\n"), size );
			goto err_memory;
		}
		filelist->Entry = (FILEENTRY *)p;
		p = (DWORD *)malloc( sizeof(DWORD) * size );
		if ( p == NULL ) {
			LOG_Put_I1(TEXT("CreateFolderNameTbl(): failed: malloc filelist->EntryIdx*%d\n"), size );
			goto err_memory;
		}
		filelist->EntryIdx = (DWORD *)p;
		filelist->EntrySize = size;
	}

	fileentrycount = 0;
	foldernametbl->EntryIdx = filelist->EntryCount;

	wsprintf( path, TEXT("%s*.*"), foldernametbl->Path );
	if (Poi_VerboseLog) LOG_Put_S1(TEXT("CreateFolderNameTbl(): [%s]\n"), path );

	while (!0) {
		if (hfind == NULL) {
			hfind = FindFirstFile( path, &finddata );
			if ( hfind == INVALID_HANDLE_VALUE ) {
				if (Poi_VerboseLog) LOG_Put_S1(TEXT("CreateFolderNameTbl(): failed: FindFirstFile [%s]\n"), path );
				goto find_failed;
			}
		}

		if (Poi_VerboseLog) LOG_Put_S2(TEXT("CreateFolderNameTbl(): [%s]: entry=[%s]\n"), path, finddata.cFileName );
		if ( !lstrcmp( finddata.cFileName, TEXT(".") ) ) {
		} else if ( !lstrcmp( finddata.cFileName, TEXT("..") ) ) {
		} else if ( finddata.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM ) {
			if (Poi_VerboseLog) LOG_Put_S2(TEXT("CreateFolderNameTbl(): [%s]: [%s] is System\n"), path, finddata.cFileName );
		} else if ( finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {
			if (Poi_VerboseLog) LOG_Put_S2(TEXT("CreateFolderNameTbl(): [%s]: [%s] is Directory\n"), path, finddata.cFileName );
			if ( findsubfolder == FALSE ) {
				goto next;
			}
			ft = (FOLDERNAMETBL *)malloc( sizeof(FOLDERNAMETBL) );
			if ( ft == NULL ) {
				LOG_Put_S1(TEXT("CreateFolderNameTbl(): [%s]: failed: malloc FOLDERNAMETBL\n"), path );
				goto err_memory;
			}
			memset( ft, 0, sizeof(FOLDERNAMETBL) );
			wsprintf( path, TEXT("%s%s\\"), foldernametbl->Path, finddata.cFileName );
			lstrcpyn( ft->Path, path, MAX_PATH );
			ft->Atr = finddata.dwFileAttributes;
			foldernametbl->SubFolder = ft;
			if ( ft_prev ) {
				ft_prev->Next = ft;
			}
			ret = CreateFolderNameTbl( ft, filelist, findsubfolder, targetext );
			if ( ret < 0 ) {
				if (Poi_VerboseLog) LOG_Put_S1I1(TEXT("CreateFolderNameTbl(): [%s]: end: ret=%d\n"), path, ret );
				return (ret);
			}
			ft_prev = ft;
		} else {
			int extmatch;
			TCHAR *extp;
			TCHAR fdir[MAX_PATH];
			TCHAR fbase[MAX_PATH];
			TCHAR fext[MAX_PATH];
			TCHAR ext[MAX_PATH+1];

			if (Poi_VerboseLog) LOG_Put_S2(TEXT("CreateFolderNameTbl(): [%s]: [%s] is File\n"), path, finddata.cFileName );
			g_ParseFileName( finddata.cFileName, fdir, fbase, fext );
			extmatch = 0;
			extp = targetext;
			while ( *(extp) != '\0' ) {
				TCHAR *p = fext;
				TCHAR *q = ext;

				if ( *(fext) == '.' ) {
					p++;
					*(q++) = '.';
				}
				while ( *(extp) != '\0' ) {
					if ( *(extp) == ',' ) {
						extp++;
						break;
					}
					if ( *(extp) == '?' ) {
						if ( *(p) != '\0' ) {
							*(q++) = *(p++);
						} else {
							*(q++) = '?';
						}
						extp++;
						continue;
					}
					if ( *(extp) == '*' ) {
						lstrcpy(q, p);
						p += lstrlen(p);
						q += lstrlen(q);
						extp++;
						continue;
					}
					*(q++) = *(extp++);
					p++;
				}
				*(q) = '\0';
				if ( !lstrcmpi( ext, fext ) ) {
					extmatch = 1;
					break;
				}
			}
			if ( !extmatch ) goto next;

			if ( filelist->EntryCount >= filelist->EntrySize ) {
				void *p;
				DWORD size;
				size = filelist->EntrySize*2;
				p = realloc( filelist->Entry, sizeof(FILEENTRY)*size );
				if ( p == NULL ) {
					LOG_Put_S1I1(TEXT("CreateFolderNameTbl(): [%s]: failed: realloc filelist->Entry*%d\n"), path, size );
					goto err_memory;
				}
				filelist->Entry = (FILEENTRY *)p;
				p = realloc( filelist->EntryIdx, sizeof(DWORD)*size );
				if ( p == NULL ) {
					LOG_Put_S1I1(TEXT("CreateFolderNameTbl(): [%s]: failed: realloc filelist->EntryIdx*%d\n"), path, size );
					goto err_memory;
				}
				filelist->EntryIdx = (DWORD *)p;
				filelist->EntrySize = size;
			}
			fileentrycount++;
			{
				DWORD c = filelist->EntryCount++;
				filelist->EntryIdx[c] = c;
				f = &filelist->Entry[c];
			}
			lstrcpyn( f->Name, finddata.cFileName, MAX_PATH );
			f->Folder = foldernametbl;
			switch (Poi_CompareTime) {
			  default:
			  case COMPARETIME_MODIFY:
				f->Time = finddata.ftLastWriteTime;
				break;

			  case COMPARETIME_CREATE:
				f->Time = finddata.ftCreationTime;
				break;

			  case COMPARETIME_ACCESS:
				f->Time = finddata.ftLastAccessTime;
				break;
			}
			f->FileSize = (((__int64)finddata.nFileSizeHigh)<<32) + finddata.nFileSizeLow;
			f->Atr = finddata.dwFileAttributes;
		}

	  next:;
		if ( !FindNextFile( hfind, &finddata ) ) {
			lasterror = GetLastError();
			if (Poi_VerboseLog) LOG_Put_S1(TEXT("CreateFolderNameTbl(): failed: FindNextFile [%s]\n"), path );
			goto find_failed;
		}
		continue;

	  find_failed:;
		{
			switch (lasterror) {
			  case ERROR_INVALID_NAME:
				LOG_Put_S1(TEXT(" tH_܂: tH_ %s\n"), foldernametbl->Path );
				goto endoffolder;

			  case ERROR_NO_MORE_FILES:
				goto endoffolder;

			  case ERROR_MORE_DATA:
				LOG_Put_S1(TEXT(" Ȃt@Ĉ߃XLbvꂽGg܂: tH_ %s\n"), foldernametbl->Path );
				// LOG_Put_S1I1(TEXT("CreateFolderNameTbl(): [%s]: FindFile: lasterror=[ERROR_MORE_DATA]\n"), path );
				goto next;

			  default:
				LOG_Put_S1I1(TEXT("CreateFolderNameTbl(): [%s]: failed: FindFile: lasterror=%d\n"), path, lasterror );
				goto err_findfile;
			}
		}
	}

  endoffolder:;
	FindClose( hfind );
	foldernametbl->FileCount = fileentrycount;
	if (Poi_VerboseLog) LOG_Put_S( TEXT("CreateFolderNameTbl(): end of folder\n") );
	return (0);

  err_quit:;
	if ( hfind ) {
		FindClose( hfind );
	}
	return (ret);

  err_memory:;
	LOG_Put_S( TEXT("CreateFolderNameTbl(): err_memory\n") );
	ret = -1;
	goto err_quit;

  err_findfile:;
	LOG_Put_S( TEXT("CreateFolderNameTbl(): err_findfile\n") );
	ret = -2;
	goto err_quit;
}

////////////////////////////////////////////////////////////////
// tH_c[J

static int FreeFolderNameTbl(
	FOLDERNAMETBL *foldernametbl
) {
	FOLDERNAMETBL *ft;
	FOLDERNAMETBL *ft_next;

	ft = foldernametbl;
	while ( ft ) {
		if ( ft->SubFolder ) {
			FreeFolderNameTbl( ft->SubFolder );
		}
		ft_next = ft->Next;
		free( ft );
		ft = ft_next;
	}

	return (0);
}

////////////////////////////////////////////////////////////////
// t@CXgJ

static int FreeFileList(
	FILELIST *filelist
) {
	if ( filelist == NULL ) {
		return (0);
	}
	if ( filelist->EntryIdx ) {
		free( filelist->EntryIdx );
	}
	if ( filelist->Entry ) {
		free( filelist->Entry );
	}
	free( filelist );

	return (0);
}

////////////////////////////////////////////////////////////////
// fBXN󂫗eʏ𓾂

static int GetDiskFreeSpaceInfo(
	TCHAR *path,
	DISKFREESPACEINFO *dfsinfo
) {
typedef BOOL (WINAPI *fp_GetDiskFreeSpaceEx)( LPCTSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER );
	fp_GetDiskFreeSpaceEx fn_GetDiskFreeSpaceEx;
	ULARGE_INTEGER l1, l2, l3;
	DWORD t1, t2, t3, t4;
	int ret;
	TCHAR drvpath[MAX_PATH];
	TCHAR c;
	int lasterror = 0;

	if (Poi_VerboseLog) LOG_Put_S1(TEXT("GetDiskFreeSpaceInfo(): start: path=[%s]\n"), path );

	__try {

#if defined(_UNICODE)
#define FUNCNAME_GetDiskFreeSpaceEx LPCSTR("GetDiskFreeSpaceExW")
#else  // defined(_UNICODE)
#define FUNCNAME_GetDiskFreeSpaceEx LPCSTR("GetDiskFreeSpaceExA")
#endif  // defined(_UNICODE)

		fn_GetDiskFreeSpaceEx = (fp_GetDiskFreeSpaceEx)GetProcAddress( GetModuleHandle(TEXT("kernel32.dll")), FUNCNAME_GetDiskFreeSpaceEx );
		if ( fn_GetDiskFreeSpaceEx ) {
			ret = fn_GetDiskFreeSpaceEx( path, &l1, &l2, &l3 );
			if ( ret == 0 ) {
				lasterror = GetLastError();
				LOG_Put_I1(TEXT("GetDiskFreeSpaceInfo(): failed: fn_GetDiskFreeSpaceEx(): lasterror=%d\n"), lasterror );
				goto err_gdfse;
			}
			dfsinfo->FreeSpace = (((__int64)l1.HighPart)<<32) + l1.LowPart;
			dfsinfo->BytePerCluster = 1024;
		} else {
			lstrcpyn( drvpath, path, 3 );
			c = tolower( drvpath[0] );
			if ( !isalpha(c) ) {
				LOG_Put_S1(TEXT("GetDiskFreeSpaceInfo(): failed: drvpath=[%s]\n"), drvpath );
				goto err_gdfs;
			}
			if ( drvpath[1] != ':' ) {
				LOG_Put_S1(TEXT("GetDiskFreeSpaceInfo(): failed: drvpath=[%s]\n"), drvpath );
				goto err_gdfs;
			}
			ret = GetDiskFreeSpace( drvpath, &t1, &t2, &t3, &t4 );
			if ( ret == 0 ) {
				lasterror = GetLastError();
				LOG_Put_I1(TEXT("GetDiskFreeSpaceInfo(): failed: GetDiskFreeSpaceEx(): lasterror=%d\n"), lasterror );
				goto err_gdfs;
			}
			dfsinfo->FreeSpace = ((__int64)t1)*t2*t3;
			dfsinfo->BytePerCluster = t1*t2;
		}
	}
	_except(EXCEPTION_EXECUTE_HANDLER)
	{
		LOG_Put_S1(TEXT("GetDiskFreeSpaceInfo(): exception: GetDiskFreeSpace(%s)\n"), path );
		return (-3);
	}

	if (Poi_VerboseLog) {
		TCHAR str[SLEN];
		_stprintf( str, TEXT("GetDiskFreeSpaceInfo(): end: FreeSpace=%I64d, BytePerCluster=%d\n"), (__int64)dfsinfo->FreeSpace, (int)dfsinfo->BytePerCluster);
		LOG_Put_S( str );
	}
	return (0);

  err_gdfse:;
	LOG_Put_S( TEXT("GetDiskFreeSpaceInfo(): err_gdfse\n"));
	return (-2);

  err_gdfs:;
	LOG_Put_S( TEXT("GetDiskFreeSpaceInfo(): err_gdfs\n"));
	return (-1);
}

////////////////////////////////////////////////////////////////
// St@CXgt\[g

static FILELIST *SortEntry_filelist;

static int __cdecl SortEntry_Compare(
	const void *p1,
	const void *p2
) {
	DWORD i1 = *((DWORD *)p1);
	FILEENTRY *e1 = &(SortEntry_filelist->Entry[i1]);
	DWORD i2 = *((DWORD *)p2);
	FILEENTRY *e2 = &(SortEntry_filelist->Entry[i2]);

	if ( e1->Time.dwHighDateTime == e2->Time.dwHighDateTime ) {
		if ( e1->Time.dwLowDateTime == e2->Time.dwLowDateTime ) {
			return (0);
		} else if ( e1->Time.dwLowDateTime < e2->Time.dwLowDateTime ) {
			return (-1);
		} else {
			return (1);
		}
	} else if ( e1->Time.dwHighDateTime < e2->Time.dwHighDateTime ) {
		return (-1);
	} else {
		return (1);
	}
}

static int SortEntry(
	FILELIST *filelist
) {
	SortEntry_filelist = filelist;
	qsort(
	  filelist->EntryIdx, 
	  filelist->EntryCount, 
	  sizeof(DWORD),
	  SortEntry_Compare
	);

	return (0);
}

////////////////////////////////////////////////////////////////
// Ƀ\[gꂽt@Cꗗ𓾂

static int GetSortedFileList(
	OPTION_FOLDER *option_folder,
	FILELIST **ret_filelist,
	FOLDERNAMETBL **ret_foldernametbl
) {
	int ret;
	FILELIST *filelist = NULL;
	FOLDERNAMETBL *foldernametbl = NULL;
	TCHAR *path;

	path = option_folder->TargetFolder;
	if (Poi_VerboseLog) LOG_Put_S1(TEXT("GetSortedFileList(): start: path=[%s]\n"), path);

	filelist = (FILELIST *)malloc(sizeof(FILELIST));
	if ( filelist == NULL ) {
		LOG_Put_S( TEXT("GetSortedFileList(): failed: malloc FILELIST\n") );
		goto err_memory;
	}
	foldernametbl = (FOLDERNAMETBL *)malloc(sizeof(FOLDERNAMETBL));
	if ( foldernametbl == NULL ) {
		LOG_Put_S( TEXT("GetSortedFileList(): failed: malloc FOLDERNAMETBL\n") );
		goto err_memory;
	}
	memset( filelist, 0, sizeof(FILELIST) );
	memset( foldernametbl, 0, sizeof(FOLDERNAMETBL) );

	lstrcpyn( foldernametbl->Path, path, MAX_PATH );
	ret = CreateFolderNameTbl(
	  foldernametbl,
	  filelist,
	  option_folder->FindSubFolder,
	  option_folder->TargetExt
	);
	if ( ret < 0 ) {
		LOG_Put_I1(TEXT("GetSortedFileList(): failed: CreateFolderNameTbl()=%d\n"), ret );
		goto err_create;
	}

	ret = SortEntry( filelist );

	*(ret_filelist) = filelist;
	*(ret_foldernametbl) = foldernametbl;

	if (Poi_VerboseLog) LOG_Put_S( TEXT("GetSortedFileList(): end\n") );
	return (0);

  err_memory:;
	ret = -1;
	LOG_Put_S( TEXT("GetSortedFileList(): err_memory\n") );
	goto err_quit;

  err_create:;
	ret = -2;
	LOG_Put_S( TEXT("GetSortedFileList(): err_create\n") );
	goto err_quit;

  err_quit:;
	if ( foldernametbl ) {
		FreeFolderNameTbl( foldernametbl );
	}
	if ( filelist ) {
		FreeFileList( filelist );
	}
	return (ret);
}

////////////////////////////////////////////////////////////////
// tH_XLċtH_폜

static int DeleteEmptyFolder(
	TCHAR *deletepath,
	int testflag
) {
	int ret;
	HANDLE hfind = NULL;
	WIN32_FIND_DATA finddata;
	int filecount;
	TCHAR path[MAX_PATH*2];
	TCHAR deletesubpath[MAX_PATH*2];
	int lasterror = 0;

	if (Poi_VerboseLog) LOG_Put_S1(TEXT("DeleteEmptyFolder(): start: deletepath=[%s]\n"), deletepath );
	filecount = 0;
	wsprintf( path, TEXT("%s*.*"), deletepath );

	while (!0) {
		if (hfind == NULL) {
			hfind = FindFirstFile( path, &finddata );
			if ( hfind == INVALID_HANDLE_VALUE ) {
				lasterror = GetLastError();
				if (Poi_VerboseLog) LOG_Put_S1(TEXT("DeleteEmptyFolder(): failed: FindFirstFile [%s]\n"), path );
				goto find_failed;
			}
		}

		if (Poi_VerboseLog) LOG_Put_S2(TEXT("DeleteEmptyFolder(): [%s]: entry=[%s]\n"), path, finddata.cFileName );
		if ( !lstrcmp( finddata.cFileName, TEXT(".") ) ) {
		} else if ( !lstrcmp( finddata.cFileName, TEXT("..") ) ) {
		} else if ( finddata.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM ) {
			if (Poi_VerboseLog) LOG_Put_S2(TEXT("DeleteEmptyFolder(): [%s]: [%s] is System\n"), path, finddata.cFileName );
			filecount++;
		} else if ( finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {
			if (Poi_VerboseLog) LOG_Put_S2(TEXT("DeleteEmptyFolder(): [%s]: [%s] is Directory\n"), path, finddata.cFileName );
			wsprintf( deletesubpath, TEXT("%s%s\\"), deletepath, finddata.cFileName );
			ret = DeleteEmptyFolder( deletesubpath, testflag );
			if ( ret < 0 ) {
				if (Poi_VerboseLog) LOG_Put_S1I1(TEXT("DeleteEmptyFolder(): [%s]: end: ret=%d\n"), path, ret );
				return (ret);
			}
			if ( ret == 0 ) {
				if (Poi_VerboseLog) LOG_Put_S2(TEXT("DeleteEmptyFolder(): [%s]: Delete Folder: %s\n"), path, deletesubpath );
				if ( !testflag ) {
					LOG_Put( TEXT(" tH_̍폜: %s\n"), deletesubpath );
					ret = RemoveDirectory( deletesubpath );
					if ( ret == 0 ) {
						LOG_Put( TEXT("  -->폜s\n"), TEXT("") );
						filecount++;
					}
				} else {
					LOG_Put( TEXT(" tH_̍폜ieXgj: %s\n"), deletesubpath );
				}
			}
		} else {
			if (Poi_VerboseLog) LOG_Put_S2(TEXT("DeleteEmptyFolder(): [%s]: [%s] is File\n"), path, finddata.cFileName );
			filecount++;
		}
		goto next;

	  next:;
		if ( !FindNextFile( hfind, &finddata ) ) {
			lasterror = GetLastError();
			if (Poi_VerboseLog) LOG_Put_S1(TEXT("DeleteEmptyFolder(): failed: FindNextFile [%s]\n"), path );
			goto find_failed;
		}
		continue;

	  find_failed:;
		{
			switch (lasterror) {
			  case ERROR_INVALID_NAME:
				LOG_Put_S1(TEXT(" tH_܂: tH_ %s\n"), deletepath );
				goto endoffolder;

			  case ERROR_NO_MORE_FILES:
				goto endoffolder;

			  case ERROR_MORE_DATA:
				LOG_Put_S1(TEXT(" Ȃt@Ĉ߃XLbvꂽGg܂: tH_ %s\n"), deletepath );
				// LOG_Put_S1I1(TEXT("DeleteEmptyFolder(): [%s]: FindFile: lasterror=[ERROR_MORE_DATA]\n"), path );
				goto next;

			  default:
				LOG_Put_S1I1(TEXT("DeleteEmptyFolder(): [%s]: failed: FindFile: lasterror=%d\n"), path, lasterror );
				goto err_findfile;
			}
		}
	}

  endoffolder:;
// LOG_Put_S1(TEXT("FindClose: %s\n"), deletepath );
	FindClose( hfind );
	if (Poi_VerboseLog) {
		LOG_Put_S( TEXT("DeleteEmptyFolder(): end of folder\n") );
		if ( testflag ) {
			TCHAR msg[SLEN];
			wsprintf( msg, TEXT(" tH_ɂt@C/tH_: %4d"), filecount );
			LOG_Put_S2(TEXT("%s (%s)\n"), msg, deletepath );
		}
	}
	return (filecount);

  err_quit:;
	if ( hfind ) {
		FindClose( hfind );
	}
	return (ret);

  err_findfile:;
	LOG_Put_S( TEXT("DeleteEmptyFolder(): err_findfile\n") );
	ret = -2;
	goto err_quit;
}

////////////////////////////////////////////////////////////////
// 폜{

static int Poi_Core(
	OPTION_FOLDER *option_folder,
	int testflag
) {
	DISKFREESPACEINFO dfs;
	int ret;
	FILELIST *filelist;
	FOLDERNAMETBL *foldernametbl;
	DWORD idx;
	DWORD delentry;
	TCHAR filename[MAX_PATH*2+1];
	__int64 needsize;
	__int64 freespace;
	__int64 filesize;
	__int64 clustersize;
	int isfirst = 1;

	if ( (!testflag) && option_folder->Stop ) {
		return (0);
	}

	if ( option_folder->VerboseLog || testflag ) {
		LOG_OSVersion();
	}

	if ( option_folder->VerboseLog || testflag ) {

		switch (option_folder->CompareTime) {
		  default:
		  case COMPARETIME_MODIFY:
			Poi_CompareTimeStr = TEXT(COMPARETIME_MODIFY_STR);
			break;

		  case COMPARETIME_CREATE:
			Poi_CompareTimeStr = TEXT(COMPARETIME_CREATE_STR);
			break;

		  case COMPARETIME_ACCESS:
			Poi_CompareTimeStr = TEXT(COMPARETIME_ACCESS_STR);
			break;
		}

		if (testflag) {
			LOG_Put( TEXT("eXgJn...\n"), TEXT("") );
		} else {
			LOG_Put( TEXT("ĎJn...\n"), TEXT("") );
		}
		LOG_Put( TEXT(" [ΏۃtH_                          : %s]\n"), option_folder->TargetFolder );
		LOG_Put( TEXT(" [Ώۊgq                            : %s]\n"), option_folder->TargetExt );
		LOG_Put( TEXT(" [rΏ                              : %s]\n"), Poi_CompareTimeStr );
		LOG_Put( TEXT(" [TutH_Ώۂɂ              : %s]\n"), (option_folder->FindSubFolder ? TEXT("͂") : TEXT("") ) );
		LOG_Put( TEXT(" [ɂȂTutH_͍폜      : %s]\n"), (option_folder->DelSubFolder ? TEXT("͂") : TEXT("") ) );
		LOG_Put( TEXT(" [ǂݎpt@C폜ΏۂɊ܂߂: %s]\n"), (option_folder->DelReadOnly ? TEXT("͂") : TEXT("") ) );
		LOG_Put( TEXT(" [Bt@C폜ΏۂɊ܂߂        : %s]\n"), (option_folder->DelHidden ? TEXT("͂") : TEXT("") ) );
		LOG_Put( TEXT(" [ڍׂȃOo͂                  : %s]\n"), (option_folder->VerboseLog ? TEXT("͂") : TEXT("") ) );
		LOG_Put_I1(TEXT(" [Kvȋ󂫗e                        : %d KoCg]\n"), option_folder->FreeSpaceSize );
	} else {
		LOG_Put( TEXT("ĎJn...\n"), TEXT("") );
	}

	ret = GetDiskFreeSpaceInfo( option_folder->TargetFolder, &dfs );
	if ( ret < 0 ) {
		LOG_Put( TEXT("fBXNeʎ擾s(%s)\n"), option_folder->TargetFolder );
		return (-1);
	}
	if ( option_folder->VerboseLog || testflag ) {
		LOG_Put( TEXT(" \n"), TEXT("") );
		LOG_Put_I1(TEXT(" ݂̋󂫗e(): %d KoCg\n"), (int)((dfs.FreeSpace+1048575)/1048576) );
		isfirst = 0;
	}


	if ( option_folder->VerboseLog || testflag ) {
		LOG_Put( TEXT(" 폜\t@Cꗗ̍쐬:\n"), TEXT("") );
	}

	ret = GetSortedFileList( option_folder, &filelist, &foldernametbl );
	if ( ret < 0 ) {
		LOG_Put( TEXT("t@CXgבւs(%s)\n"), option_folder->TargetFolder );
		return (-2);
	}

	if ( option_folder->VerboseLog || testflag ) {
		LOG_Put( TEXT(" 폜\t@C̏ڍ:\n"), TEXT("") );
		LOG_Put( TEXT(" iɂقǐɍ폜܂j\n"), TEXT("") );
		idx = 0;
		while ( idx < filelist->EntryCount ) {
			SYSTEMTIME ti;
			FILETIME fi;
			TCHAR datestr[SLEN];
			TCHAR timestr[SLEN];
	
			delentry = filelist->EntryIdx[idx];
	
			wsprintf(
			  filename, TEXT("%s%s"), 
			  filelist->Entry[delentry].Folder->Path, 
			  filelist->Entry[delentry].Name
			);
	
			FileTimeToLocalFileTime( &filelist->Entry[delentry].Time, &fi );
			FileTimeToSystemTime( &fi, &ti );
			GetDateFormat( NULL, 0, &ti, TEXT("yyyy/MM/dd"), datestr, SLEN );
			GetTimeFormat( NULL, 0, &ti, TEXT("HH:mm:ss"), timestr, SLEN );
			LOG_Put_S3(TEXT(" %s %s : %s\n"), datestr, timestr, filename );
	
			idx++;
		}
	}

	if ( option_folder->VerboseLog || testflag ) {
		LOG_Put( TEXT(" 폜ɂ󂫗eʂ̕ϓ:\n"), TEXT("") );
	}
	clustersize = dfs.BytePerCluster;
	freespace = dfs.FreeSpace;
	idx = 0;
	while ( idx < filelist->EntryCount ) {
		needsize = ((__int64)(option_folder->FreeSpaceSize))*1024*1024;
		if ( freespace >= needsize ) {
			if ( !testflag ) {
				break;
			}
		}

		if ( isfirst ) {
			LOG_Put_I1(TEXT(" ݂̋󂫗e(): %d KoCg\n"), (int)((dfs.FreeSpace+1048575)/1048576) );
			isfirst = 0;
		}

		delentry = filelist->EntryIdx[idx];

		wsprintf(
		  filename, TEXT("%s%s"), 
		  filelist->Entry[delentry].Folder->Path, 
		  filelist->Entry[delentry].Name
		);

		if ( filelist->Entry[delentry].Atr & FILE_ATTRIBUTE_READONLY ) {
			if ( option_folder->DelReadOnly ) {
				ret = TRUE;
				if ( !testflag ) {
					ret = SetFileAttributes( filename, FILE_ATTRIBUTE_NORMAL );
				}
				if ( ret == FALSE ) {
					LOG_Put_S1(TEXT(" ݋֎~̉: %s\n"), filename );
					LOG_Put( TEXT("  --> s\n"), TEXT("") );
					goto next;
				}
			} else {
				LOG_Put_S1(TEXT(" ݋֎~ɂXLbv: %s\n"), filename );
				goto next;
			}
		}

		if ( filelist->Entry[delentry].Atr & FILE_ATTRIBUTE_HIDDEN ) {
			if ( option_folder->DelHidden ) {
				// no operation
			} else {
				LOG_Put_S1(TEXT(" BɂXLbv: %s\n"), filename );
				goto next;
			}
		}

		ret = TRUE;
		if ( !testflag ) {
			LOG_Put_S1(TEXT(" 폜: %s\n"), filename );
#if 1
			__try {
				ret = DeleteFile( filename );
			}
			_except(EXCEPTION_EXECUTE_HANDLER)
			{
				ret = FALSE;
			}
#else
			ret = TRUE;
#endif
		} else {
			LOG_Put_S1(TEXT(" 폜ieXgj: %s\n"), filename );
		}
		if ( ret == FALSE ) {
			LOG_Put( TEXT("  --> s\n"), TEXT("") );
			goto next;
		} else {
			filesize = filelist->Entry[delentry].FileSize;
			filesize = (filesize+clustersize-1)/clustersize*clustersize;
			freespace += filesize;
		}

		LOG_Put_I1(TEXT(" 폜̋󂫗e(\): %d KoCg\n"), (int)((freespace+1048575)/1048576) );

	  next:;
		idx++;
	}
	FreeFolderNameTbl( foldernametbl );
	FreeFileList( filelist );

	if ( option_folder->DelSubFolder ) {
		if ( option_folder->VerboseLog || testflag ) {
			LOG_Put( TEXT(" 폜\tH_̈ꗗ:\n"), TEXT("") );
		}
		ret = DeleteEmptyFolder( option_folder->TargetFolder, testflag );
	}

	if ( testflag ) {
		LOG_Put( TEXT("eXgI...\n"), TEXT("") );
	} else {
		LOG_Put( TEXT("ĎI...\n"), TEXT("") );
	}

	ret = GetDiskFreeSpaceInfo( option_folder->TargetFolder, &dfs );
	if ( ret < 0 ) {
		LOG_Put( TEXT("fBXNeʎ擾s(%s)\n"), option_folder->TargetFolder );
		return (-3);
	}
	if ( !isfirst ) {
		LOG_Put_I1(TEXT("݂̋󂫗e(): %d KoCg\n"), (int)((dfs.FreeSpace+1048575)/1048576) );
	}

	return (0);
}

////////////////////////////////////////////////////////////////
// 폜Xbh

static DWORD WINAPI Thread_Poi_Core(
	LPVOID param
) {
	HANDLE hmutex;
	POITHREADPARAM *ptp;

	ptp = (POITHREADPARAM *)param;

	// Z}tH擾
	hmutex = CreateMutex( NULL, FALSE, TEXT("frpoi_Thread_PoiCore") );
	if ( hmutex == NULL ) {
		return (-1);
	}
	Poi_Busy = 1;
	WaitForSingleObject( hmutex, INFINITE );

	Poi_VerboseLog = ptp->option->VerboseLog;
	Poi_CompareTime = ptp->option->CompareTime;
	Poi_Core( ptp->option, ptp->testflag );

	CloseHandle( hmutex );

	if ( ptp->hwndnotify ) {
		SendMessage( ptp->hwndnotify, WM_USER_MAIN_ENDPOI, 0, 0 );
	}
	Poi_Busy = 0;

	GlobalFree( ptp );
	return (0);
}

////////////////////////////////////////////////////////////////
// 폜Xbh

int Fork_Poi_Main(
	OPTION_FOLDER *option,
	int testflag,
	HWND hwndnotify
) {
	HANDLE thread;
	DWORD threadid;
	POITHREADPARAM *ptp;

	ptp = (POITHREADPARAM *)GlobalAlloc( GPTR, sizeof(POITHREADPARAM) );
	if ( ptp == NULL ) {
		return (-1);
	}
	ptp->option = option;
	ptp->testflag = testflag;
	ptp->hwndnotify = hwndnotify;

	thread = CreateThread(
	  NULL,
	  0,
	  (LPTHREAD_START_ROUTINE)Thread_Poi_Core,
	  (LPDWORD)ptp,
	  0,
	  &threadid
	);
	if ( thread ) {
		CloseHandle( thread );
	}

	return (0);
}

////////////////////////////////////////////////////////////////
// [EOF]
