// wp_gettestlog: get test log
// Copyright (C) 2004 GORRY.

#include "wp_gettestlog.h"

static HWND MY_HWND;

////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
// UNICODEpRICHEDITeLXgݒ

#if defined(_UNICODE)

typedef struct tagEditStreamText {
	LPCWSTR txptr;
	DWORD lenwcs;
} EditStreamText;

DWORD CALLBACK EditStreamCallBack(
  EditStreamText *est,
  LPBYTE buf,
  LONG bufbytes,
  LONG *retbytes
) {
	if (est->lenwcs == 0) {
		*(retbytes) = 0;
	} else {
		DWORD bufwcs = bufbytes / sizeof(WCHAR);
		DWORD copywcs = min(est->lenwcs, (DWORD)bufwcs);
		DWORD copybytes = copywcs*sizeof(WCHAR);
		memcpy(buf, est->txptr, copybytes);
		est->lenwcs -= copywcs;
		est->txptr += copywcs;
		*(retbytes) = copybytes;
	}
	return (0);
}

void RichEdit_SetTextW(HWND hwnd, LPARAM lparam)
{
	EditStreamText est;
	EDITSTREAM es;
	LPCWSTR text = (LPCWSTR)lparam;

	est.txptr = text;
	est.lenwcs = wcslen(text);
	es.dwCookie = (DWORD_PTR)&est;
	es.dwError = 0;
	es.pfnCallback = (EDITSTREAMCALLBACK)EditStreamCallBack;
	SendMessage(hwnd, EM_STREAMIN, (WPARAM)IntToPtr(SF_TEXT|SF_UNICODE), (LPARAM)&es);
}

void RichEdit_ReplaceSelW(HWND hwnd, LPARAM lparam)
{
	EditStreamText est;
	EDITSTREAM es;
	LPCWSTR text = (LPCWSTR)lparam;

	est.txptr = text;
	est.lenwcs = wcslen(text);
	es.dwCookie = (DWORD_PTR)&est;
	es.dwError = 0;
	es.pfnCallback = (EDITSTREAMCALLBACK)EditStreamCallBack;
	SendMessage(hwnd, EM_STREAMIN, (WPARAM)(SF_TEXT|SF_UNICODE|SFF_SELECTION), (LPARAM)&es);
}

#endif  // defined(_UNICODE)

////////////////////////////////////////////////////////////////
// CvV[W

static BOOL CALLBACK MY_PROC(
	HWND hwnd,
	UINT message,
	WPARAM wParam,
	LPARAM lParam
) {
	WPParam wp;
	WORD id, cmd;
	BOOL result = 0;
	HWND hctl;
	int margins;
	int w, h;
	RECT rect_window;
	RECT rect_ok;

	wp.hwnd = hwnd;
	wp.message = message;
	wp.wParam = wParam;
	wp.lParam = lParam;

	switch ( message ) {
	  case WM_INITDIALOG:
		MY_HWND = hwnd;
		g_CenteringWindow( hwnd, NULL );
		hctl = GetDlgItem( hwnd, IDOK );
		EnableWindow( hctl, FALSE );
		hctl = GetDlgItem( hwnd, IDC_RICHEDIT_LOG );
		SendMessage( hctl, EM_SETTEXTMODE, TM_PLAINTEXT, 0 );
		SendMessage( hwnd, WM_SIZE, 0, 0 );
		HWND_PutLog = hwnd;
		Fork_Poi_Main( (OPTION_FOLDER *)lParam, 1, hwnd );
		break;

	  case WM_USER_MAIN_ENDPOI:
		hctl = GetDlgItem( hwnd, IDOK );
		EnableWindow( hctl, TRUE );
		break;

	  case WM_APP_LOGMSG:
		hctl = GetDlgItem( hwnd, IDC_RICHEDIT_LOG );
		margins = SendMessage( hctl, EM_GETMARGINS, 0, -1 );
		Edit_SetSel( hctl, -1, -1 );
#if defined(_UNICODE)
		RichEdit_ReplaceSelW( hctl, lParam );
#else  // defined(_UNICODE)
		Edit_ReplaceSel( hctl, lParam );
#endif  // defined(_UNICODE)
		Edit_ScrollCaret( hctl );
		return (TRUE);

	  case WM_COMMAND:
		id = GET_WM_COMMAND_ID( wParam, lParam );
		cmd = GET_WM_COMMAND_CMD( wParam, lParam );
		switch (id) {
		  case IDOK:
			EndDialog( hwnd, 1 );
			return (TRUE);

		  case IDHELP:
//			WinHelp( MY_HWND, HLP_FILE, HELP_CONTEXT, HELP_ID );
			return (TRUE);

		  case IDCANCEL:
			EndDialog( hwnd, 0 );
			return (TRUE);
		}
		break;

	  case WM_SIZE:
		GetWindowRect( hwnd, &rect_window );
		GetWindowRect( GetDlgItem(hwnd, IDOK), &rect_ok );
		rect_ok.right -= rect_ok.left;
		rect_ok.bottom -= rect_ok.top;
		w = rect_window.right - rect_window.left
		                      - (GetSystemMetrics(SM_CXSIZEFRAME)*2);
		h = rect_window.bottom - rect_window.top
		                       - (GetSystemMetrics(SM_CYSIZEFRAME)*2)
		                       - rect_ok.bottom*12/10
		                       - GetSystemMetrics(SM_CYCAPTION);
		MoveWindow(
		  GetDlgItem(hwnd, IDC_RICHEDIT_LOG),
		  0, 0, w, h,
		  TRUE
		);
		MoveWindow(
		  GetDlgItem(hwnd, IDOK),
		  w-(rect_ok.right*12/10),
		  h+rect_ok.bottom/10,
		  rect_ok.right, rect_ok.bottom,
		  TRUE
		);

		break;

	  case WM_DESTROY:
		HWND_PutLog = NULL;
		MY_HWND = NULL;
		break;

	}
	return (result);
}

////////////////////////////////////////////////////////////////
// EBhE쐬

int WP_GetTestLog_Dialog(
	HWND hwnd,
	OPTION_FOLDER *option
) {
	int ret;

	ret = DialogBoxParam( ThisInst, MAKEINTRESOURCE(IDD_GETTESTLOG), hwnd, MY_PROC, (LPARAM)option );
	return (ret);
}

////////////////////////////////////////////////////////////////
// [EOF]
